;Global variables
ButtonsToButtons.GlobalShiftingFlag  := 0 ; available to all instances, lets us know if another instance is currently in shift mode.
	
	
class ButtonsToButtons extends _UCR.Classes.Plugin {
	Type := "Remapper (Buttons To Buttons)"
	Description := "Remaps buttons as well as a shift button to create another set of virtual buttons"

	LocalShiftingFlag := 0     ; Use this and the GlobalShiftingFlag to know whether this or other instances 
										; are in shift mode and avoid holding down 2 buttons at once if possible.
	
	NumberOfButtons := 20


	Init(){  ; Create the GUI
			
		;Create Primary shift button
		Gui, Add, GroupBox, Center xm ym w165 h60 section, Shift Button
		this.AddControl("InputButton", "IB1", 0, this.MyHkChangedState.Bind(this), "xs+5 yp+15")
		this.AddControl("ButtonPreview", "", 0, this.IOControls.IB1, "x+5 yp+5")
				
		;Each group of buttons requires its own loop because they position relative to each other.	
		Loop % this.NumberOfButtons {  ; Create Input Buttons
			if(A_Index==1) {
				Gui, Add, GroupBox, Center x175 ym w165 h760	section, Input Button        .
				this.AddControl("InputButton", "IB" + A_Index+1, 0, this.MyHkChangedState.Bind(this), "xs+5 yp+15")
				this.AddControl("ButtonPreview", "", 0, this.IOControls["IB" A_Index+1], "x+5 yp+5")			
			}else{
				this.AddControl("InputButton", "IB" + A_Index+1, 0, this.MyHkChangedState.Bind(this), "xs+5 yp+32")
				this.AddControl("ButtonPreview", "", 0, this.IOControls["IB" A_Index+1], "x+5 yp+5")
			}	
		}
	

		Loop % this.NumberOfButtons {   ; Create Non-Shifted Output Buttons
			if(A_Index==1) {
				Gui, Add, GroupBox, Center x340 ym w170 h760 section, Non-Shifted Output Button            .
				this.AddControl("OutputButton", "OB" + A_Index, 0, "xs+5 yp+15")
				this.AddControl("ButtonPreview", "", 0, this.IOControls["OB" A_Index], "x+5 yp+5")
			}else{
				this.AddControl("OutputButton", "OB" + A_Index, 0, "xs+5 yp+32")
				this.AddControl("ButtonPreview", "", 0, this.IOControls["OB" A_Index], "x+5 yp+5")
			}
		}
			
		Loop % this.NumberOfButtons {   ; Create Shifted Output Buttons
			if(A_Index==1) {		
				Gui, Add, GroupBox, Center x510 ym w170 h760 section, Shifted Output	Button           .
				this.AddControl("OutputButton", "OB" + A_Index+this.NumberOfButtons, 0, "xs+5 ys+15")
				this.AddControl("ButtonPreview", "", 0, this.IOControls["OB" A_Index+this.NumberOfButtons], "x+5 yp+5")
			}else{
				this.AddControl("OutputButton","OB" + A_Index+this.NumberOfButtons, 0, "xs+5 yp+32")
				this.AddControl("ButtonPreview", "", 0, this.IOControls["OB" A_Index+this.NumberOfButtons], "x+5 yp+5")
			}
		}
				
		;Checkbox. If another copy of this plugin is loaded, disable non-shifted button if other plugin shift key is pressed.
		Gui, Add, GroupBox, Center xm ym+60 w165 h65 section, Settings
		this.AddControl("Checkbox", "Toggle", 0, "xs+5 yp+30", "Disable Non-Shift output if")				
		Gui, Add, Text, y+1, % "      another plugin is Shifted"
		
		;Clear the controls , otherwise they sometimes initialize with a '-1'	
		loop % this.NumberOfButtons + 1 {
		this.IOControls["IB" A_Index].Set(0)
		}
	
	}
	
	; The main code is here. Called when a key or button changes state (pressed or released)
	MyHkChangedState(e){
		;OutputDebug, % "UCR| Plugin" this.Name " changed state to: " (e ? "Down" : "Up")		
		
		if(this.IOControls.IB1.Get()){ ; Shift key is down so produce Shifted Button output
			this.ClearNonShiftedOutputs()
			this.UpdateShiftedOutputs()
			ButtonsToButtons.GlobalShiftingFlag  := 1
			this.LocalShiftingFlag := 1
		}else {	
		
			;If "Disable Non-Shifted" is checked, apply rule to non-shifted output		
			if(this.GuiControls.Toggle.Get()){ 
														  

				;If no other shift, or my shift key is pressed
				if(ButtonsToButtons.GlobalShiftingFlag == 0 || this.LocalShiftingFlag == 1) { 
					this.ClearShiftedOutputs()
					this.UpdateNonShiftedOutputs()
					ButtonsToButtons.GlobalShiftingFlag  := 0
					this.LocalShiftingFlag := 0			
				}else
				{
					this.ClearShiftedOutputs()
					this.ClearNonShiftedOutputs()
					this.LocalShiftingFlag := 0					
				}
								
			}else{
					;If "Disable Non-Shifted" is not checked, just output non-shifted 
						this.ClearShiftedOutputs()
						this.UpdateNonShiftedOutputs()
						ButtonsToButtons.GlobalShiftingFlag  := 0
						this.LocalShiftingFlag := 0
			}
		}
	}
	
	ClearNonShiftedOutputs(){
			Loop % this.NumberOfButtons
			this.IOControls["OB" A_Index].Set(0)
	}

	UpdateNonShiftedOutputs(){
			Loop % this.NumberOfButtons 
			this.IOControls["OB" A_Index].Set(this.IOControls["IB" A_Index+1].Get())
	}

	ClearShiftedOutputs(){
			Loop % this.NumberOfButtons
			this.IOControls["OB" A_Index+this.NumberOfButtons].Set(0)
	}

	UpdateShiftedOutputs(){
			Loop % this.NumberOfButtons 
			this.IOControls["OB" A_Index+this.NumberOfButtons].Set(this.IOControls["IB" A_Index+1].Get())
	}
	
}
